function [trees, crossValErrors] = accelTreesCreation(matrixFinal, ystr, algInd, orderSFS, name, respName, numPoints, lastPoint, indPartDay)
%function [trees, crossValErrors] = accelTreesCreation(matrixFinal, ystr, algInd, orderSFS, name, respName, numPoints, lastPoint, indPartDay)
%DESCRIPTION: Create three Decision Trees
%INPUTS:
%   - matrixFinal: Matrix where each column contains a predicting value and
%   each row is the associated instance
%   - ystr: Vector with instances (for each observation)
%   - algInd: Indices for Training
%   - orderSFS: Ordered features according to SFS
%   - name: Name of the features
%   - respName: Name of the tree
%   - numPoints: Number of points in the logspace to find the optimal MinLeaf
%   - lastPoint: Number used to divide the number of observations (to get the final point)
%   - indPartDay: Index part day feature
%RETURN:
%   - trees: Three types of trees
%   - crossValErrors: Cross Validation Errors of each model
%Author: Bernat Carbons
%Modification Date: 25/06/2014

rng(8000,'twister');
matrixSFS = matrixFinal(algInd, orderSFS);
nameSFS = name(orderSFS);
ystrSFS = ystr(algInd);

%Logical Vector for categorical predictor: partDay
catPred=false(1, length(orderSFS));
numCat=0;
if ~isempty(find(orderSFS==indPartDay, 1))
    catPred(orderSFS==indPartDay)=true;
    numCat=3; %0, 1 and 2
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%1. Split Criterion->Gini's Diversity Index. Prune Criterion->Impurity
treeSFS1 = fitctree(matrixSFS,ystrSFS,'SplitCriterion','gdi', ...
    'PruneCriterion','impurity', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName);


%Select Appropiate Tree Depth. 'MinLeaf' parameter. Cross Validation 
leafs = unique(round(logspace(1,log10(lastPoint),numPoints)));

N = numel(leafs);
err1 = zeros(N,1);
for n=1:N
    t = fitctree(matrixSFS,ystrSFS,'SplitCriterion','gdi', ...
    'PruneCriterion','impurity', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
    'CrossVal','On','MinLeaf',leafs(n));%CrossValidation tree with 10 folds
    err1(n) = kfoldLoss(t);
end
figure;
plot(leafs,err1);
xlabel('Min Leaf Size');
ylabel('cross-validated error');
title('Split Criterion->Ginis Diversity Index. Prune Criterion->Impurity');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%2. Split Criterion->Maximum Deviance Reduction (Cross Entropy). Prune Criterion->Impurity
treeSFS2 = fitctree(matrixSFS,ystrSFS,'SplitCriterion','deviance', ...
    'PruneCriterion','impurity', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName);

%Select Appropiate Tree Depth. 'MinLeaf' parameter. Cross Validation 
N = numel(leafs);
err2 = zeros(N,1);
for n=1:N
    t = fitctree(matrixSFS,ystrSFS,'SplitCriterion','deviance', ...
    'PruneCriterion','impurity', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
    'CrossVal','On','MinLeaf',leafs(n));%CrossValidation tree with 10 folds
    err2(n) = kfoldLoss(t);
end
figure;
plot(leafs,err2);
xlabel('Min Leaf Size');
ylabel('cross-validated error');
title('Split Criterion->Maximum Deviance Reduction (Cross Entropy). Prune Criterion->Impurity');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%3. Split Criterion->Twoing Rule. Prune Criterion->Node error
treeSFS3 = fitctree(matrixSFS,ystrSFS,'SplitCriterion','twoing', ...
    'PruneCriterion','error', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName);

%Select Appropiate Tree Depth. 'MinLeaf' parameter. Cross Validation
N = numel(leafs);
err3 = zeros(N,1);
for n=1:N
    t = fitctree(matrixSFS,ystrSFS,'SplitCriterion','twoing', ...
    'PruneCriterion','error', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
    'CrossVal','On','MinLeaf',leafs(n));%CrossValidation tree with 10 folds
    err3(n) = kfoldLoss(t);
end
figure;
plot(leafs,err3);
xlabel('Min Leaf Size');
ylabel('cross-validated error');
title('Split Criterion->Twoing Rule. Prune Criterion->Node error');


%Comparison
figure;
plot(leafs,err1,'r',leafs,err2,'b',leafs, err3, 'g');
title('Comparison of the Split and Prune Criterions');
xlabel('Min Leaf Size');
ylabel('cross-validated error');
legend('SC: Ginis. PC: Impurity','SC: Deviance. PC: Impurity','SC: Twoing. PC: Node error');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%CHOOSING THE SUITABLE MIN-LEAF FOR ALL THE TREES
leafOpt=input('Min Leaf Size: ');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Plot the default and the optimal trees
%1. Split Criterion->Gini's Diversity Index. Prune Criterion->Impurity
OptTreeSFS1 = fitctree(matrixSFS,ystrSFS,'SplitCriterion','gdi', ...
    'PruneCriterion','impurity', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
    'MinLeaf',leafOpt);

view(treeSFS1,'mode','graph')
view(OptTreeSFS1,'mode','graph')

%Get the Cross Validation Error
crossValErrors(1) = kfoldLoss(crossval(OptTreeSFS1,'KFold',10));

%2. Split Criterion->Maximum Deviance Reduction (Cross Entropy). Prune Criterion->Impurity
OptTreeSFS2 = fitctree(matrixSFS,ystrSFS,'SplitCriterion','deviance', ...
    'PruneCriterion','impurity', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
    'MinLeaf',leafOpt);

view(treeSFS2,'mode','graph')
view(OptTreeSFS2,'mode','graph')

%Get the Cross Validation Error
crossValErrors(2) = kfoldLoss(crossval(OptTreeSFS2,'KFold',10));

%3. Split Criterion->Twoing Rule. Prune Criterion->Node error
OptTreeSFS3 = fitctree(matrixSFS,ystrSFS,'SplitCriterion','twoing', ...
    'PruneCriterion','error', 'CategoricalPredictors',catPred, ...
    'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
    'MinLeaf',leafOpt);

view(treeSFS3,'mode','graph')
view(OptTreeSFS3,'mode','graph')

%Get the Cross Validation Error
crossValErrors(3) = kfoldLoss(crossval(OptTreeSFS3,'KFold',10));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trees{1}=OptTreeSFS1;
trees{2}=OptTreeSFS2;
trees{3}=OptTreeSFS3;


