function [infoObs, infoQuest] = testModel(matrixFinal, y, algInd, tunInd, testInd, orderSFS, model, numObsQuest, optionModel, dataActivityMets, SVMparam)
%function [infoObs, infoQuest] = testModel(matrixFinal, y, algInd, tunInd, testInd, orderSFS, model, numObsQuest, optionModel, dataActivityMets, SVMparam)
%DESCRIPTION: Test the final model
%accuracy
%INPUTS:
%   - matrixFinal: Matrix where each column contains a predicting value and
%   each row is the associated instance
%   - y: Vector with instances (for each observation). Identifiers
%   - algInd: Indices for Training
%   - tunInd: Indices for Validation
%   - orderSFS: Ordered features according to SFS
%   - model: struct containing information of the model
%   - numObsQuest: Number of observations for questionnaire
%   - optionModel: Type of model: 1->Tree, 2->NB, 3->SVM
%   - dataActivityMets: All possible activities, their identifiers and METs
%   - SVMparam: Parameters of the SVM models. 0 Otherwise
%RETURN:
%   - infoObs: Struct containing: Mean and standard deviation of accuracy.
%   Also the accuracy for each class. OBSERVATIONS (10 s)
%   - infoQuest: Struct containing: Mean and standard deviation of accuracy
%   in observations and questionnaires. QUESTIONNAIRES
%Author: Bernat Carbons
%Modification Date: 05/07/2014


%Set random
rng(8000,'twister');

%Get the METs associated to each activity
mets = dataActivityMets{4}; %MET's burned per hour

if optionModel~=3
    %Select the desired rows and columns of the final matrix
    matrixTest = matrixFinal(testInd, orderSFS);
    matrixTun = matrixFinal(tunInd, orderSFS);
    matrixTrain = matrixFinal(algInd, orderSFS);
else
    matrixTest = matrixFinal(testInd, :);
    matrixTun = matrixFinal(tunInd, :);
    matrixTrain = matrixFinal(algInd, :);
end

%Join the matrix
matrixVal=[matrixTrain;matrixTun;matrixTest];

%Identifier of the activities for observations (numeric array)
name_idObs=unique(y);
name_idQuest=unique(y);

%Transform ystr indices to identifier. Easier calculations.
[y, name_idQuestN]=grp2idx(y);

%Get the classes
yRealTest = y(testInd);
yRealTun = y(tunInd);
yRealTrain = y(algInd);

%Join the classes
yRealVal=[yRealTrain;yRealTun;yRealTest];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PART 1. CROSS VALIDATION.
numParts=5;
sizeParts=(length(algInd)+length(tunInd)+length(testInd))/numParts;

%Indices of each part
indX=1:1:numParts;
indX=(indX-1)*sizeParts+1;

%Divide Data in five parts in order to make a crossValidation
c=cvpartition(yRealVal(indX), 'kFold', numParts);

trainIdx=cell(1,numParts);
testIdx=cell(1, numParts);

for in=1:numParts
    aux=indX(training(c,in));
    for ini=1:numParts-1
       trainIdx{in}=[trainIdx{in},aux(ini):1:aux(ini)+sizeParts-1];
    end
    aux=indX(test(c,in));
    testIdx{in}=aux:1:aux+sizeParts-1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Predict the values
yPredict=cell(1,numParts);
yReal=cell(1,numParts);

switch optionModel
    case 1 %Decision Tree
        
        %Create different trees using different data. The paramaters are the same
        splitCrit=model.ModelParameters.SplitCriterion;
        pruneCrit=model.ModelParameters.PruneCriterion;
        catPred=model.CategoricalPredictors;
        numCat=model.ModelParameters.MaxCat;
        nameSFS=model.PredictorNames;
        respName=model.ResponseName;
        mleaf=model.ModelParameters.MinLeaf;
        
        for in=1:numParts
            %Create the 'training' matrix
            matrixTree=matrixVal(trainIdx{in},:);
            
            %Create the 'clases' vector
            ynumTree=yRealVal(trainIdx{in});
            
            %Create a tree
            model= fitctree(matrixTree,ynumTree,'SplitCriterion',splitCrit, ...
                'PruneCriterion',pruneCrit, 'CategoricalPredictors',catPred, ...
                'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
                'MinLeaf',mleaf);
            
            
            %Predict the values
            yPredict{in}=predict(model,matrixVal(testIdx{in},:));
                    
            %Real values
           yReal{in}=yRealVal(testIdx{in});
    
        end
        
    case 2 %Naive Bayes
        
        %Create different NaiveBayes using different data. The paramaters are the same
        distNB=model.Dist;
        if strcmp(distNB{1},'normal')
            kstypeNB=[];
            kswidthNB=[];
        else
            kstypeNB=model.Params{1}.Kernel;
            kswidthNB=zeros(1,length(distNB));
            for in=1:length(distNB)
                if ~isnumeric(model.Params{1,in})
                    kswidthNB(in)=model.Params{1,in}.BandWidth;
                end
            end
        end

        
        for in=1:numParts
            %Create the 'training' matrix
            matrixNB=matrixVal(trainIdx{in},:);
            
            %Create the 'clases' vector
            ynumNB=yRealVal(trainIdx{in});
            
            %Create a tree
            model= fitNaiveBayes(matrixNB,ynumNB,'Distribution',distNB, ...
                'KSType', kstypeNB, 'KSWidth', kswidthNB);
            
            
            %Predict the values
            yPredict{in}=predict(model,matrixVal(testIdx{in},:));
            
            %Real values
            yReal{in}=yRealVal(testIdx{in});

            

            
        end
        
        
    case 3 %SVM
                  
        %Create different SVMs using different data. The paramaters are the same
        kernelType=SVMparam.KernelType;
        constr=SVMparam.OptimalConstraint;
        gam=0;
        coef0=0;
        
        if kernelType==2||kernelType==1 %RBF or Polynomial
            gam=SVMparam.OptimalGamma;
            if kernelType==1 %polynomial
                coef0=SVMparam.optimalCoef0;
            end
        end
        
        for in=1:numParts
            %Create the 'training' matrix
            matrixSVM=matrixVal(trainIdx{in},:);
            
            %Create the 'clases' vector
            ynumSVM=yRealVal(trainIdx{in});
            
            %Create a SVM
            model= svmtrain(ynumSVM, matrixSVM,['-t',32,num2str(kernelType),32,'-c', 32, num2str(constr), 32, '-g', 32, num2str(gam),32,'-r', 32, num2str(coef0),32 '-d 2 -q']);
               
            %Predict the values
            yPredict{in}=svmpredict([1:sizeParts]',matrixVal(testIdx{in},:),model);
            
            %Real values
            yReal{in}=yRealVal(testIdx{in});
            
        end         
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Compute the Confusion Matrix
%OBSERVATIONS (10s)

%Average
accuracyObs=zeros(1, numParts);
accMetsObs=zeros(1, numParts);
metsAbsErrorObs=zeros(1, numParts);

confusionsObs=cell(1,numParts);
orderObs=cell(1,numParts);
%Compute confusion matrix for each part (5)
for in=1:numParts
    [confusionsObs{in}, orderObs{in}]=confusionmat(yReal{in},yPredict{in});
    accuracyObs(in)=trace(confusionsObs{in})/(sum(sum(confusionsObs{in})));
    
    %Compute the accuracy in METs. Mean
    yPredict{in}(isnan(yPredict{in}))=0;
    yPredict{in}(yPredict{in}==0)=length(mets);%Find not trained index. Last position
    metsP=10/3600*mets(yPredict{in});
    metsR=10/3600*mets(yReal{in});
    
    accMetsObs(in)=1-mean(abs((metsR-metsP)./metsR));
    
    %Absolute error. MET/h Mean
    metsAbsErrorObs(in)=3600/10*mean(abs(metsP-metsR));
end

%Results
avgAccObs=mean(accuracyObs)
stdAccObs=std(accuracyObs);
avgAccMetsObs=mean(accMetsObs)
stdAccMetsObs=std(accMetsObs);
avgMetsAbsErrorObs=mean(metsAbsErrorObs)
stdMetsAbsErrorObs=std(metsAbsErrorObs);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Compute the Confusion Matrix
%QUESTIONNAIRES (15min)

%Split data. Windows size=numObsQuest
YpredictM=cell(1,numParts);
yRealM=cell(1,numParts);


for in=1:numParts
    %Predicted values
    YpredictM{in}=buffer(yPredict{in},numObsQuest);
    %Real values
    yRealM{in}=buffer(yReal{in},numObsQuest);
    yReal{in}=yRealM{in}(1,:);
    
end

[~,numCols]=size(YpredictM{1});

for n=1:numParts  
    yPredict{n}=zeros(1,numCols);
    for in=1:numCols
        aux=unique(YpredictM{n}(:,in));
        %Get the different activities
        c=zeros(1,length(aux));
        for ini=1:length(aux)
            c(ini)=length(find(YpredictM{n}(:,in)==aux(ini)));
        end
        %Get the confidence interval
        confIntervalPredict=max(c)/numObsQuest;
        
        %Condition: (1 class) or (single max and confidence interval
        %minimum 40% and difference among two maximums 20%)
        if length(c)==1 || (length(find(max(c)==c))==1)&&(confIntervalPredict>=0.4) && (confIntervalPredict-(max(c(c~=max(c)))/numObsQuest)>0.2)
            %Get the maximum number and the selected activity over the 15 min of questionnaire
            yPredict{n}(in)=aux(max(c)==c);
        else
            yPredict{n}(in)=0;%No trained
        end
    end
end



%Average
accuracyQuest=zeros(1, numParts);
accMetsQuest=zeros(1,numParts);
metsAbsErrorQuest=zeros(1,numParts);

confusionsQuest=cell(1,numParts);
orderQuest=cell(1,numParts);
%Compute confusion matrix for each part (4)
for in=1:numParts
    [confusionsQuest{in}, orderQuest{in}]=confusionmat(yReal{in},yPredict{in}); 
    accuracyQuest(in)=trace(confusionsQuest{in})/(sum(sum(confusionsQuest{in})));
    
    %Compute the accuracy in METs. Mean
    yPredict{in}(yPredict{in}==0)=length(mets); %Find not trained index. Last position
    metsP=15/60*mets(yPredict{in});
    metsR=15/60*mets(yReal{in});
    
    accMetsQuest(in)=1-mean(abs((metsR-metsP)./metsR));
    
    %Absolute error. MET/h Mean
    metsAbsErrorQuest(in)=60/15*mean(abs(metsP-metsR));
end

%Results
avgAccQuest=mean(accuracyQuest)
stdAccQuest=std(accuracyQuest);
avgAccMetsQuest=mean(accMetsQuest)
stdAccMetsQuest=std(accMetsQuest);
avgMetsAbsErrorQuest=mean(metsAbsErrorQuest)
stdMetsAbsErrorQuest=std(metsAbsErrorQuest);









%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PART 2. CLASS ACCURACY

%OBSERVATIONS (10s)

%Average
confFinalObs=zeros(length(name_idObs));

%Fill the confusions matrix with zeros the missing classes to average
for in=1:length(confusionsObs)
    numMis=setdiff(1:length(name_idObs),orderObs{in});
    if ~isempty(numMis)
        sizeIni=length(name_idObs)-length(numMis);
        for ini=1:length(numMis)
            %Column
            confusionsObs{in}=[confusionsObs{in}(:,1:numMis(ini)-1), ...
                zeros(sizeIni,1),confusionsObs{in}(:,numMis(ini):end)];
            
            sizeIni=sizeIni+1;
            
            %Row
            confusionsObs{in}=[confusionsObs{in}(1:numMis(ini)-1,:); zeros(1,sizeIni); ...
                            confusionsObs{in}(numMis(ini):end,:)];

        end   
    end
    
    %Average
    confFinalObs=confFinalObs+confusionsObs{in};
end


%Find the zero element columns -> Not trained enough
zeroCol=find(~any(confFinalObs));
confFinalObs(:,zeroCol)=[];
confFinalObs(zeroCol,:)=[];

name_idObs(zeroCol)=[];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%QUESTIONNAIRES (15min)


name_idQuestN=['Not trained';name_idQuestN];
%Average
confFinalQuest=zeros(length(name_idQuestN));

%Fill the confusions matrix with zeros the missing classes to average
for in=1:length(confusionsQuest)
    numMis=setdiff(0:length(name_idQuestN)-1,orderQuest{in});
    if ~isempty(numMis)
        numMis=numMis+1;
        sizeIni=length(name_idQuestN)-length(numMis);
        for ini=1:length(numMis)
            %Column
            confusionsQuest{in}=[confusionsQuest{in}(:,1:numMis(ini)-1), ...
                zeros(sizeIni,1),confusionsQuest{in}(:,numMis(ini):end)];
            
            sizeIni=sizeIni+1;
            
            %Row
            confusionsQuest{in}=[confusionsQuest{in}(1:numMis(ini)-1,:); zeros(1,sizeIni); ...
                            confusionsQuest{in}(numMis(ini):end,:)];

        end   
    end
    
    %Average
    confFinalQuest=confFinalQuest+confusionsQuest{in};
end

%Find the zero element columns -> Not trained enough
zeroCol=find(~any(confFinalQuest));
confFinalQuest(:,zeroCol)=[];
confFinalQuest(zeroCol,:)=[];

name_idQuestN(zeroCol)=[];
name_idQuest(zeroCol-1)=[];







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %PART 3. METs ERROR
% 
% %OBSERVATIONS (10s)
% %Compute the Real METs burned for each class. 
% metsRealObs=10/3600*sum(confFinalObs').*mets(name_idObs)';
% 
% %Compute the predicted METs burned for each class
% metsPredictObs=10/3600*confFinalObs*mets(name_idObs);
% 
% %Compute the Relative Error for each class
% relErrorObs=abs((metsRealObs-metsPredictObs')./metsRealObs);
% 
% %Compute the Absolute Error for each class
% absErrorObs=abs(metsRealObs-metsPredictObs');
% 
% %Compute the average and the standard deviation for all classes jointly
% avgRelErrorObs=mean(relErrorObs);
% stdRelErrorObs=std(relErrorObs);
% 
% avgAbsErrorObs=mean(absErrorObs);
% stdAbsErrorObs=std(absErrorObs);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %QUESTIONNAIRES (15min)
% 
% mets=[0;mets]; %Not trained
% name_idQuest=name_idQuest+1;
% name_idQuest=[1;name_idQuest];%not trained
% 
% %Compute the Real METs burned for each class
% metsRealQuest=15/60*sum(confFinalQuest').*mets(name_idQuest)';
% metsRealQuest(1)=[];%Not trained
% 
% %Compute the predicted METs burned for each class
% metsPredictQuest=15/60*confFinalQuest*mets(name_idQuest);
% metsPredictQuest(1)=[];%Not trained
% 
% %Compute the Relative Error for each class
% relErrorQuest=abs((metsRealQuest-metsPredictQuest')./metsRealQuest);
% 
% %Compute the Absolute Error for each class
% absErrorQuest=abs(metsRealQuest-metsPredictQuest');
% 
% %Compute the average and the standard deviation for all classes jointly
% avgRelErrorQuest=mean(relErrorQuest);
% stdRelErrorQuest=std(relErrorQuest);
% 
% avgAbsErrorQuest=mean(absErrorQuest);
% stdAbsErrorQuest=std(absErrorQuest);
% 
% 
% 
% 
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PART 4. RESULTS

%OBSERVATIONS


%Compute the accuracy for each class
name_idObs
classAccObs=diag(confFinalObs)'./sum(confFinalObs')

infoObs=struct('MeanAccuracy',avgAccObs,'StandardDeviationAccuracy', ...
    stdAccObs,'ClassNames',{name_idObs},'ClassAccuracy',classAccObs, ...
    'MeanRelativeAccMET', avgAccMetsObs, 'StandardDeviationRelativeAccMET', ...
    stdAccMetsObs, 'MeanAbsoluteErrorMET',avgMetsAbsErrorObs, ...
    'StandardDeviationAbsoluteErrorMET',stdMetsAbsErrorObs);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%QUESTIONNAIRES

%Compute the accuracy for each class
classAccQuest=diag(confFinalQuest)'./sum(confFinalQuest');

%Do not include 'Not trained'
name_idQuestN=name_idQuestN(2:end)
classAccQuest(1)=[];

classAccQuest


infoQuest=struct('MeanAccuracy',avgAccQuest,'StandardDeviationAccuracy', ...
    stdAccQuest,'ClassNames',{name_idQuestN},'ClassAccuracy',classAccQuest, ...
    'MeanRelativeAccMET', avgAccMetsQuest, 'StandardDeviationRelativeAccMET', ...
    stdAccMetsQuest, 'MeanAbsoluteErrorMET',avgMetsAbsErrorQuest, ...
    'StandardDeviationAbsoluteErrorMET',stdMetsAbsErrorQuest)


