function [SVMs, SVMsParam, crossValErrors ] = accelSVMCreation(matrixScaled, ynum, algInd)
%function [SVMs, SVMsParam, crossValErrors] = accelSVMCreation(matrixScaled, ynum, algInd)
%DESCRIPTION: Support Vector Machine Creation
%INPUTS:
%   - matrixScaled: Scaled matrix (from -1 to 1)
%   - ynum: Vector with instances (for each observation) in numbers 
%   - algInd: Indices for Training
%RETURN:
%   - SVMs: Three types of SVMs
%   - SVMsParam: Parameters of the SVM models
%   - crossValErrors: Cross Validation Errors of each model
%Author: Bernat Carbons
%Modification Date: 26/06/2014

%Get the desired values of the matrix
matrixScaled = matrixScaled(algInd, :);
ynumSFS = ynum(algInd);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MODELS - One-vs-One (OVO)

%1. kernel->Radial Basis function

%Constraint Parameter
staC=5;
endC=17;
constraint1=staC:2:endC;
numPointsC=length(constraint1);

%Gamma Parameter
staG=-3;
endG=3;
gam1=staG:2:endG;
numPointsG=length(gam1);

%Cross Validation to obtain the optimal Constraint and Gamma
accuracy1=zeros(numPointsC,numPointsG);

for in=1:numPointsC
    for ini=1:numPointsG
        accuracy1(in,ini) = svmtrain(ynumSFS, matrixScaled,['-t 2 -c', 32, num2str(2^constraint1(in)), 32, '-g', 32, num2str(2^gam1(ini)),32, '-v 3 -10']);
    end
end

%PLOT
figure;
color=['k','b', 'y', 'r', 'g','m','c',':','-.','--'];
hold on;
legendStr=cell(1,numPointsG);
for in=1:numPointsG
    plot(constraint1, accuracy1(:,in),color(in));
    legendStr{in}=strcat('Gamma = 2^',num2str(gam1(in)));
end

set(gca, 'XTick', constraint1);
for j = 1:length(constraint1)
    xtl{j} = ['2^' num2str(constraint1(j))];
end
set(gca, 'XTickLabel', xtl)

legend(legendStr);
xlabel('Constraint (C)')
ylabel('Cross Validation Accuracy')
title('Cross Validation Accuracy. Gamma vs Constraint')


%Optimal values
[optC1, optG1]=find(max(max(accuracy1))==accuracy1,1);
SVMs{1} = svmtrain(ynumSFS, matrixScaled,['-t 2 -c', 32, num2str(2^constraint1(optC1)), 32, '-g', 32, num2str(2^gam1(optG1)),32, '-q']);
SVMsParam{1}=struct('KernelType',2,'OptimalConstraint',2^constraint1(optC1),'OptimalGamma',2^gam1(optG1));


%Cross Validation Error
crossValErrors(1)=(100-max(max(accuracy1)))/100



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%2. kernel->linear

%Constraint Parameter
staC=7;
endC=11;
constraint2=staC:2:endC;
numPointsC=length(constraint2);


%Cross Validation to obtain the optimal Constraint
accuracy2=zeros(1,numPointsC);

for in=1:numPointsC
    accuracy2(in) = svmtrain(ynumSFS, matrixScaled,['-t 0 -c', 32, num2str(2^constraint2(in)), 32, '-v 10 -q']); 
end



%PLOT
figure;
plot(constraint2,accuracy2);
xlabel('Constraint (C)')
ylabel('Cross Validation Accuracy')
title('Cross Validation Accuracy. Constraint')
set(gca, 'XTick', constraint2);
for j = 1:length(constraint2)
  xtl{j} = ['2^' num2str(constraint2(j))];
end
set(gca, 'XTickLabel', xtl)

%Optimal values
optC2=find(max(accuracy2)==accuracy2);
SVMs{2} = svmtrain(ynumSFS, matrixScaled,['-t 0 -c', 32, num2str(2^constraint2(optC2)), 32, '-q']);
SVMsParam{2}=struct('KernelType', 0,'OptimalConstraint', 2^constraint2(optC2));

%Cross Validation Error
crossValErrors(2)=(100-max(max(accuracy2)))/100



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%3. kernel->polynomial
%Constraint
staC=7;
endC=15;
constraint3=staC:2:endC;
numC=length(constraint3);

%Gamma
staG=-3;
endG=3;
gam3=staG:2:endG;
numG=length(gam3);

%Coef0
coef3=[0,1];
numCoef=length(coef3);


accuracy3=zeros(numCoef,numC*numG);

for indCoef=1:numCoef
    for indG=1:numG
        for indC=1:numC
             accuracy3(indCoef,(indG-1)*numC+indC) = svmtrain(ynumSFS, matrixScaled,['-t 1 -c', 32, num2str(2^constraint3(indC)), 32, '-g', 32, num2str(2^gam3(indG)),32,'-r', 32, num2str(coef3(indCoef)),32 '-d 2',32, '-v 10 -q']);
        end
    end
end


%PLOT
color=['k','b', 'y', 'r', 'g','m','c',':','-.','--'];
for indCoef=1:length(coef3)
    figure;
    hold on;
    legendStr=cell(1,numG);
    for indG=1:numG
        plot(constraint3, accuracy3(indCoef, ...
            ((indG-1)*numC+1):(indG*numC)),color(indG));
        legendStr{indG}=strcat('Gamma = 2^',num2str(gam3(indG)));
    end
    
    set(gca, 'XTick', constraint3);
    for j = 1:length(constraint3)
        xtl{j} = ['2^' num2str(constraint3(j))];
    end
    set(gca, 'XTickLabel', xtl)
    
    legend(legendStr);
    xlabel('Constraint (C)')
    ylabel('Cross Validation Accuracy')
    title(strcat('Coef0:',32, num2str(coef3(indCoef)),32, ...
        '.Cross Validation Accuracy. Gamma vs Constraint'))
end

%Optimal values in the accuracy matrix
[optRow, optCol]=find(max(max(accuracy3))==accuracy3,1);

%Obtain the optimal parameters
optG3=ceil(optCol/numC);
optC3=optCol-(optG3-1)*numC;
optCoef3=optRow;
SVMs{3} = svmtrain(ynumSFS, matrixScaled,['-t 1 -c', 32, num2str(2^constraint3(optC3)), 32, '-g', 32, num2str(2^gam3(optG3)),32,'-r', 32, num2str(coef3(optCoef3)),32 '-d 2',32, ' -q']);
SVMsParam{3}=struct('KernelType',1,'OptimalConstraint', 2^constraint3(optC3), 'OptimalGamma',2^gam3(optG3),'OptimalCoef0',coef3(optCoef3));

%Cross Validation Error
crossValErrors(3)=(100-max(max(accuracy3)))/100




















