function infoAcc = validateModel(matrixFinal, y, algInd, tunInd, orderSFS, model, numObsQuest, optionModel, SVMparam)
%function infoAcc = validateModel(matrixFinal, y, algInd, tunInd, orderSFS, model, numObsQuest, optionModel, SVMparam)
%DESCRIPTION:Validate the model computing the confusion matrix and its
%accuracy (average and standard deviation)
%INPUTS:
%   - matrixFinal: Matrix where each column contains a predicting value and
%   each row is the associated instance
%   - y: Vector with instances (for each observation)
%   - algInd: Indices for Training
%   - tunInd: Indices for Validation
%   - orderSFS: Ordered features according to SFS
%   - model: struct containing information of the model
%   - numObsQuest: Number of observations for questionnaire
%   - optionModel: Type of model: 1->Tree, 2->NB, 3->SVM
%   - SVMparam: Parameters of the SVM model. 0 otherwise
%RETURN:
%   - infoAcc: Struct containing: Mean and standard deviation of accuracy
%   in observations and questionnaires
%Author: Bernat Carbons
%Modification Date: 04/07/2014


%Set random
rng(8000,'twister');


%Transform ystr indices to identifier. Easier calculations.
if optionModel~=3 %SVM y=num already.
    y=grp2idx(y);
    %Select the desired rows and columns of the final matrix
    matrixTun = matrixFinal(tunInd, orderSFS);
    matrixTrain = matrixFinal(algInd, orderSFS);
else
    matrixTun = matrixFinal(tunInd, :);
    matrixTrain = matrixFinal(algInd, :);
end
    
%Join the matrix
matrixVal=[matrixTrain;matrixTun];

%Get the classes
yRealTun = y(tunInd);
yRealTrain = y(algInd);

%Join the classes
yRealVal=[yRealTrain;yRealTun];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%CROSS VALIDATION.
numParts=4;
sizeParts=(length(algInd)+length(tunInd))/numParts;

%Indices of each part
indX=1:1:numParts;
indX=(indX-1)*sizeParts+1;

%Divide Data in four parts in order to make a crossValidation
c=cvpartition(yRealVal(indX), 'kFold', numParts);

trainIdx=cell(1,numParts);
testIdx=cell(1, numParts);

for in=1:numParts
    aux=indX(training(c,in));
    for ini=1:numParts-1
       trainIdx{in}=[trainIdx{in},aux(ini):1:aux(ini)+sizeParts-1];
    end
    aux=indX(test(c,in));
    testIdx{in}=aux:1:aux+sizeParts-1;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Predict the values
yPredict=cell(1,numParts);
yReal=cell(1,numParts);


switch optionModel
    case 1 %Decision Tree
        
        %Create different trees using different data. The paramaters are the same
        splitCrit=model.ModelParameters.SplitCriterion;
        pruneCrit=model.ModelParameters.PruneCriterion;
        catPred=model.CategoricalPredictors;
        numCat=model.ModelParameters.MaxCat;
        nameSFS=model.PredictorNames;
        respName=model.ResponseName;
        mleaf=model.ModelParameters.MinLeaf;
        
        for in=1:numParts
            %Create the 'training' matrix
            matrixTree=matrixVal(trainIdx{in},:);
            
            %Create the 'clases' vector
            ynumTree=yRealVal(trainIdx{in});
            
            %Create a tree
            model= fitctree(matrixTree,ynumTree,'SplitCriterion',splitCrit, ...
                'PruneCriterion',pruneCrit, 'CategoricalPredictors',catPred, ...
                'MaxCat',numCat,'PredictorNames',nameSFS,'ResponseName', respName, ...
                'MinLeaf',mleaf);
            
            
            %Predict the values
            yPredict{in}=predict(model,matrixVal(testIdx{in},:));
                    
            %Real values
           yReal{in}=yRealVal(testIdx{in});

     
        end
        
    case 2 %Naive Bayes
        
        %Create different NaiveBayes using different data. The paramaters are the same
        distNB=model.Dist;
        if strcmp(distNB{end},'normal')
            kstypeNB=[];
            kswidthNB=[];
        else
            kstypeNB=model.Params{end}.Kernel;
            kswidthNB=zeros(1,length(distNB));
            for in=1:length(distNB)
                if ~isnumeric(model.Params{end,in})
                    kswidthNB(in)=model.Params{end,in}.BandWidth;
                end
            end
        end

        
        for in=1:numParts
            %Create the 'training' matrix
            matrixNB=matrixVal(trainIdx{in},:);
            
            %Create the 'clases' vector
            ynumNB=yRealVal(trainIdx{in});
            
            %Create a Naive Bayes
            model= fitNaiveBayes(matrixNB,ynumNB,'Distribution',distNB, ...
                'KSType', kstypeNB, 'KSWidth', kswidthNB);
            
            
            %Predict the values
            yPredict{in}=predict(model,matrixVal(testIdx{in},:));
            
            %Real values
            yReal{in}=yRealVal(testIdx{in});

            

            
        end
        
        
    case 3 %SVM
                  
        %Create different SVMs using different data. The paramaters are the same
        kernelType=SVMparam.KernelType;
        constr=SVMparam.OptimalConstraint;
        gam=0;
        coef0=0;
        
        if kernelType==2||kernelType==1 %RBF or Polynomial
            gam=SVMparam.OptimalGamma;
            if kernelType==1 %polynomial
                coef0=SVMparam.optimalCoef0;
            end
        end
        
        for in=1:numParts
            %Create the 'training' matrix
            matrixSVM=matrixVal(trainIdx{in},:);
            
            %Create the 'clases' vector
            ynumSVM=yRealVal(trainIdx{in});
            
            %Create a SVM
            model= svmtrain(ynumSVM, matrixSVM,['-t',32,num2str(kernelType),32,'-c', 32, num2str(constr), 32, '-g', 32, num2str(gam),32,'-r', 32, num2str(coef0),32 '-d 2 -q']);
               
            %Predict the values
            yPredict{in}=svmpredict([1:sizeParts]',matrixVal(testIdx{in},:),model);
            
            %Real values
            yReal{in}=yRealVal(testIdx{in});
            
        end
        
        
        
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Compute the Confusion Matrix
%OBSERVATIONS (10s)

%Average
accuracyObs=zeros(1, numParts);

%Compute confusion matrix for each part (4)
for in=1:numParts
    confusionObs=confusionmat(yReal{in},yPredict{in});
    accuracyObs(in)=trace(confusionObs)/(sum(sum(confusionObs)));
end

%Results
accuracyObs
avgAccObs=mean(accuracyObs)
stdAccObs=std(accuracyObs)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Compute the Confusion Matrix
%QUESTIONNAIRES (15min)


%Split data. Windows size=numObsQuest
YpredictM=cell(1,numParts);
yRealM=cell(1,numParts);

for in=1:numParts
    %Predicted values
    YpredictM{in}=buffer(yPredict{in},numObsQuest);
    %Real values
    yRealM{in}=buffer(yReal{in},numObsQuest);
    yReal{in}=yRealM{in}(1,:);
    
end

[~,numCols]=size(YpredictM{1});

for n=1:numParts  
    yPredict{n}=zeros(1,numCols);
    for in=1:numCols
        aux=unique(YpredictM{n}(:,in));
        %Get the different activities
        c=zeros(1,length(aux));
        for ini=1:length(aux)
            c(ini)=length(find(YpredictM{n}(:,in)==aux(ini)));
        end
        %Get the confidence interval
        confIntervalPredict=max(c)/numObsQuest;
        
        %Condition: (1 class) or (single max and confidence interval
        %minimum 40% and difference among two maximums 20%)
        if length(c)==1 || (length(find(max(c)==c))==1)&&(confIntervalPredict>=0.4) && (confIntervalPredict-(max(c(c~=max(c)))/numObsQuest)>0.2)
            %Get the maximum number and the selected activity over the 15 min of questionnaire
            yPredict{n}(in)=aux(max(c)==c);
        else
            yPredict{n}(in)=0;%No trained
        end
    end
end



%Average
accuracyQuest=zeros(1, numParts);

%Compute confusion matrix for each part (4)
for in=1:numParts
    confusionQuest=confusionmat(yReal{in},yPredict{in});
    accuracyQuest(in)=trace(confusionQuest)/(sum(sum(confusionQuest)));
end

%Results
accuracyQuest
avgAccQuest=mean(accuracyQuest)
stdAccQuest=std(accuracyQuest)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
infoAcc=struct('obsMeanAccuracy',avgAccObs,'obsStandardDeviationAccuracy', ...
    stdAccObs,'questMeanAccuracy',avgAccQuest,'questStandardDeviationAccuracy', ...
    stdAccQuest);


