function [dist, v, h, tdd, date]=gpsData(fs, file, pathname, optPlot, optPopFile)
%function [dist, v, h, tdd, date]=gpsData(fs, file, pathname, optPlot, optPopFile)
%DESCRIPTION: Compute the velocity using GPS data. Plot the velocity,
%distance and altitude in function of time
%INPUTS:
%   - fs: Sample frequency
%   - file: Name of the file
%   - pathname: Path of the file
%   - optPlot: 0-->No plot. 1-->Plot
%   - optPopFile: 0-->Set file and pathname. 1--> Pop to chose the desired
%   file
%RETURN:
%   - dist: Distance in meters
%   - v: Velocity in km/h
%   - h: Altitude in meters
%   - tdd: Normalized timeStamp in seconds
%   - date: Start date data. Format equal SQL. ex: 20140219181611
%Author: Bernat Carbons
%Modification Date: 09/05/2014



%Load the GPSData file

if optPopFile
    
    [file, pathname]=uigetfile('*.txt');
end

fileComplete = fullfile(pathname, file);


filetxt = regexprep(file, '_', '-');

%Date
date=regexprep(filetxt,'GPSData-','');
date=regexprep(date,'-','');
date=regexprep(date,'.txt','');
date=str2double(date)+2e13;
 
%Import data from the GPSData.......txt
[~,Latitud,Longitud,Altitud,Accuracy,timeLong1] = importfileGPSData(fileComplete);
t=(timeLong1-timeLong1(1))*1e-3;

%Resample data
[newData, t]=resampleData([Latitud,Longitud,Altitud,Accuracy],t,fs);
Latitud=newData(:,1);
Longitud=newData(:,2);
Altitud=newData(:,3);
Accuracy=newData(:,4);

t0=gethour(file); %Get the start time in hours. t0=8+40/60+1/3600;

Rtierra=40e3*1000/2/pi; % Radius of the earth in meters
x=(Longitud-Longitud(1))/180*pi*Rtierra; %Transform the longitude from degrees to meters and normalize
y=(Latitud-Latitud(1))/180*pi*Rtierra; %Transform the latitude from degrees to meters and normalize
%z=Altitud-Altitud(1); %Normalize the altitude
tant=t(1); %Save the start time
nant=1;
n=1;
dist(1)=0;


%Compute the altitude, the distance and the speed
for k=1:length(t)
    if (t(k)-tant)>60 %Time larger than 1 min.
        d(n)=sqrt((x(k)-x(nant)).^2+(y(k)-y(nant))^2);
        if d(n)>Accuracy(k)%%&&(d(n)/(t(k)-tant))*3.6<140
            h(n)=Altitud(k);
            %dt(n)=t(k)-tant;
            dt(n)=t(k)-t(nant);
            tdd(n)=t(k);
            nant=k;
            n=n+1;
            dist(n)=dist(n-1)+d(n-1);
        else
            d=d(1:n-1);
        end
        tant=t(k);
    end
end
dist=dist(1:n-1);

%Still
if numel(d)==0
    tdd=t;
    dt=t;
    d=zeros(1,length(t));
    h=1/length(Altitud)*(ones(1,length(Altitud)))*Altitud;
    dist=d;
end
 

%PLOTS 
 
%Plot the velocity from GPS data
if optPlot
    figure;
    subplot(3,1,1);
    plot(tdd/3600+t0,(d./dt)*3.6);
    grid;
    xlabel('Time (h)');
    title(['Velocity from GPS data, file: ' filetxt]);
    ylabel('Velocity (km/h)');
    v=(d./dt)*3.6;
    
    %Plot the altitude from GPS data
    subplot(3,1,3);
    plot(tdd/3600+t0,h);
    grid;
    xlabel('Time (h)');
    title(['Altitude from GPS data, file: ' filetxt]);
    ylabel('Altitude (m)');
    
    %Plot the distance from GPS data
    subplot(3,1,2);
    plot(tdd/3600+t0,dist/1000);
    grid;
    xlabel('Time (h)');
    title(['Distance from GPS data, file: ' filetxt]);
    ylabel('Distance (km)');
    
    
    %Plot the velocity and the distance from GPS data
    figure;
    AX=plotyy(tdd/3600+t0,(d./dt)*3.6,tdd/3600+t0,dist/1000);
    grid;
    xlabel('Time (h)');
    title(['Velocity and Distance from GPS data, file: ' filetxt]);
    set(get(AX(1),'Ylabel'),'String','Velocity (km/h)');
    set(get(AX(2),'Ylabel'),'String','Distance (km)')
 
end
